/*
Weather Station - G2C 3G Demo Project

    Date          : March 2020
    Author        : Nemanja Medakovic

Test configuration STM32 :

    MCU              : STM32F407ZG
    MCU Clock        : 168MHz
    Dev. Board       : UNI CLICKER
    ARM Compiler ver : v6.2.0.0

---

Description :

The application is composed of three sections :

- System Initialization - Initializes all necessary GPIO pins, UART module to 
  communicate with G2C 3G click and I2C used for the communication with sensors.
- Application Initialization - Initializes all sensors drivers, performs a 
  default configurations for the all sensors.
  All Click boards will be initialized in this function.
- Application Task - Weather results will be sent to ClickCloud web application 
  every one second if they were changed.
  Weather Station can measure temperature, humidity and ambient light.
  This project also consists informations about time and date.
  Actuator enable/disable, in the ClickCloud application, is used to control 
  results updating on the ClickCloud.
  
- Additional functions -

- g2c3g_actuatorDemo - Actuator parser for this demo example.
- g2c3g_defaultHandler - Default handler for this demo example.

***** Note:
- Items required before starting the program:
1. Set operator parameters( APN, username and password ) for the SIM card.
2. Create a device on the ClickCloud and set the device parameters
   ( device_key and device_password ).
3. Enter sensor and/or actuator reference from manifest associated
   with the created device.
- You can find the ClickCloud web application at the following link:
https://app.clickcloud.io/#/login

*/

#include "Click_G2C3G_types.h"
#include "Click_G2C3G_config.h"
#include "Click_G2C3G_timer.h"
#include "Click_Ambient6_types.h"
#include "Click_Ambient6_config.h"
#include "Click_RTC_7_types.h"
#include "Click_TempHum8_types.h"
#include "cstring.h"


/** 
 * ClickCloud actuator and sensors reference
 *
 * NOTE: By changing this parameters user can modify the actuator and sensor 
 * reference.
 */
static char g2c_actuatorRef[ 10 ] = "RTC7_EN";

static char g2c_tempRef [ 15 ] = "TEMPHUM8_T";
static char g2c_humRef  [ 15 ] = "TEMPHUM8_H";
static char g2c_lightRef[ 15 ] = "AMBIENT6_LX";
static char g2c_hourRef [ 10 ] = "RTC7_H";
static char g2c_minRef  [ 10 ] = "RTC7_MIN";
static char g2c_weekRef [ 10 ] = "RTC7_WEEK";
static char g2c_dayRef  [ 10 ] = "RTC7_DAY";
static char g2c_monthRef[ 15 ] = "RTC7_MONTH";
static char g2c_yearRef [ 10 ] = "RTC7_YEAR";

/** Operator settings */
static char operator_apn      [ 15 ] = "vipmobile";
static char operator_username [ 15 ] = "vipmobile";
static char operator_password [ 15 ] = "vipmobile";

/** Device cloud settings */
static char device_key      [ 20 ] = "dox8zkrv5jc3p809";
static char device_password [ 50 ] = "9e81b16b-c76f-47f4-afdb-9ce48eeb5b25";

/** Control variables to enable/disable data update */
static uint8_t state;
static uint8_t oldstate;

/** G2C 3G click commands */
static char G2C3G_AT     [ 3 ] = "AT";
static char G2C3G_ATE    [ 5 ] = "ATE0";
static char G2C3G_AT_GMR [ 7 ] = "AT+GMR";
static char G2C3G_AT_CEN [ 9 ] = "AT+CEN=1";
static char G2C3G_AT_NWC [ 9 ] = "AT+NWC=1";
static char G2C3G_AT_BRC [ 9 ] = "AT+BRC=1";
static char G2C3G_AT_PUB [ 7 ] = "AT+PUB";

/** 
 * Data where stored the previous measurement results
 * (needed for data update comparison) 
 */
static rtc7_time_t prev_time;
static int8_t      prev_temp;
static uint8_t     prev_hum;
static uint16_t    prev_light;

/** Data measurement results */
static rtc7_time_t time_set;
static rtc7_time_t time_date;
static int8_t      temperature;
static uint8_t     humidity;
static uint16_t    ambient_light;


static void g2c3g_actuatorDemo( char *rsp )
{
    uint8_t sw_state;
    uint8_t err = G2C3G_PARSER_NO_ERROR;

    /** Actuator switcher parser */
    err = g2c3g_actuator_sw_parser( rsp, &g2c_actuatorRef[ 0 ], &sw_state );
    
    if ( err == G2C3G_PARSER_NO_ERROR )
    {
        state = sw_state;
    }
}

static void g2c3g_defaultHandler( char *rsp, uint8_t *evArgs )
{
    g2c3g_actuatorDemo( rsp );
}

void systemInit( )
{
    /** G2C 3G click init */
    mikrobus_gpioInit( _MIKROBUS4, _MIKROBUS_AN_PIN,  _GPIO_INPUT );
    mikrobus_gpioInit( _MIKROBUS4, _MIKROBUS_RST_PIN, _GPIO_OUTPUT );
    mikrobus_gpioInit( _MIKROBUS4, _MIKROBUS_PWM_PIN, _GPIO_INPUT );
    mikrobus_uartInit( _MIKROBUS4, &G2C3G_UART_CFG[ 0 ] );

    if ( G2C3G_POLL_ENABLE != 1 )
    {
        g2c3g_uart_int_cfg( );
    }

    /** GPIO initialization for RTC 7 click */
    mikrobus_gpioInit( _MIKROBUS3, _MIKROBUS_INT_PIN, _GPIO_INPUT );
    mikrobus_gpioInit( _MIKROBUS3, _MIKROBUS_AN_PIN,  _GPIO_INPUT );
    mikrobus_gpioInit( _MIKROBUS3, _MIKROBUS_RST_PIN, _GPIO_INPUT );

    /** I2C initialization for first three mikrobus sockets */
    mikrobus_i2cInit( _MIKROBUS2, &_AMBIENT6_I2C_CFG[ 0 ] );
    Delay_ms( 300 );
}

void applicationInit( )
{
    /** Temp-Hum 8 init */
    temphum8_i2cDriverInit( (T_TEMPHUM8_P)&_MIKROBUS1_GPIO,
                            (T_TEMPHUM8_P)&_MIKROBUS1_I2C, 0x40 );
    Delay_ms( 300 );
    temphum8_softReset( );
    Delay_ms( 200 );
    temphum8_configuration( _TEMPHUM8_CFG_RESOLUTION_H12_T14 |
                            _TEMPHUM8_CFG_ENABLE_ON_CHIP_HEATER | 
                            _TEMPHUM8_CFG_DISABLE_OTP_RELOAD );

    /** Ambient 6 init */
    ambient6_i2cDriverInit( (T_AMBIENT6_P)&_MIKROBUS2_GPIO, 
                            (T_AMBIENT6_P)&_MIKROBUS2_I2C, 
                            _AMBIENT6_DEVICE_SLAVE_ADDRESS );
    Delay_ms( 300 );
    ambient6_configuration( _AMBIENT6_CFG_ALS_SENSITIVITY_X2 |
                            _AMBIENT6_CFG_ALS_TIME_SETTING_25MS |
                            _AMBIENT6_CFG_PERSISTENCE_NUMBER_1 |
                            _AMBIENT6_CFG_ALS_INT_DISABLE |
                            _AMBIENT6_CFG_ALS_POWER_ON );
    ambient6_writeData( _AMBIENT6_CMD_MODE_SETTINGS, _AMBIENT6_POWER_MODE_1 | 
                        _AMBIENT6_POWER_MODE_ENABLE );

    /** RTC 7 init */
    /** 
     * If you want to reset RTC 7 click on the selected time, modify and use
     * this time configuration 
     */
    time_set.seconds  = 0;
    time_set.minutes  = 0;
    time_set.hours    = 10;
    time_set.weekdays = 5;
    time_set.monthday = 20;
    time_set.month    = 3;
    time_set.year     = 20;

    rtc7_i2cDriverInit( (T_RTC7_P)&_MIKROBUS3_GPIO, (T_RTC7_P)&_MIKROBUS3_I2C, 
                        0x69 );
    Delay_ms( 300 );
    rtc7_reset( );
    Delay_ms( 200 );
    rtc7_disableDst( );
    rtc7_init( 0, _RTC7_24HR_FORMAT );
    rtc7_setGmtTime( &time_set );
    rtc7_setOsc( _RTC7_ENABLE_OSC, _RTC7_INPUT_FREQ_32768HZ, 
                 _RTC7_OUTPUT_FREQ_32768HZ );
    rtc7_setTimer( _RTC7_TIMER_EN, _RTC7_TIMER_FREQ_16HZ );

    /** G2C 3G TIMER INIT */
    g2c3g_timer_int_cfg( );

    /** G2C 3G DRIVER INIT */
    g2c3g_uart_driver_init( (g2c3g_obj_t)&_MIKROBUS4_GPIO,
                            (g2c3g_obj_t)&_MIKROBUS4_UART );
    g2c3g_core_init( g2c3g_defaultHandler, 1500 );

    /** G2C 3G MODULE POWER ON */
    g2c3g_module_power( G2C3G_MODULE_POWER_ON );
    g2c3g_reset_module( );
    Delay_ms( 3000 );

    while ( g2c3g_device_status( ) != G2C3G_STATUS_DEVICE_READY );

    g2c3g_send_cmd( &G2C3G_AT[ 0 ] );
    g2c3g_send_cmd( &G2C3G_AT_GMR[ 0 ] );
    g2c3g_send_cmd( &G2C3G_AT_CEN[ 0 ] );
    g2c3g_send_operator_cfg( &operator_apn[ 0 ],
                             &operator_username[ 0 ],
                             &operator_password[ 0 ] );
    g2c3g_send_cmd( &G2C3G_AT_NWC[ 0 ] );
    
    while ( g2c3g_network_status( ) != 1 );

    g2c3g_send_broker_cfg( &device_key[ 0 ], &device_password[ 0 ] );
    g2c3g_send_cmd( &G2C3G_AT_BRC[ 0 ] );

    while ( g2c3g_broker_status( ) != 1 );
    Delay_ms( 1000 );

    state    = 1;
    oldstate = 0;

    prev_time.seconds  = 60;
    prev_time.minutes  = 60;
    prev_time.hours    = 24;
    prev_time.weekdays = 0;
    prev_time.monthday = 0;
    prev_time.month    = 0;
    prev_time.year     = 100;
    prev_temp          = 200;
    prev_hum           = 101;
    prev_light         = 65535;
}

void applicationTask( )
{
    /** CORE STATE MACHINE */
    g2c3g_process( );

    /** START MEASUREMENT */
    if ( ( state == 1 ) && ( oldstate == 0 ) )
    {
        prev_time.seconds  = 60;
        prev_time.minutes  = 60;
        prev_time.hours    = 24;
        prev_time.weekdays = 0;
        prev_time.monthday = 0;
        prev_time.month    = 0;
        prev_time.year     = 100;
        prev_temp          = 200;
        prev_hum           = 101;
        prev_light         = 65535;

        oldstate = 1;
    }

    /** CLOUD MEASUREMENT RESULTS UPDATE */
    if ( ( state == 1 ) && ( oldstate == 1 ) )
    {
        rtc7_getLocalTime( &time_date );

        if ( prev_time.seconds != time_date.seconds )
        {
            char tx_tmp[ 20 ];

            /** TIME UPDATE */
            prev_time.seconds = time_date.seconds;
            
            if ( prev_time.hours != time_date.hours )
            {
                prev_time.hours = time_date.hours;
                
                ByteToStr( time_date.hours, tx_tmp );
                Ltrim( tx_tmp );
                
                g2c3g_send_data_ref( &g2c_hourRef[ 0 ], &tx_tmp[ 0 ] );
                g2c3g_send_cmd( &G2C3G_AT_PUB[ 0 ] );
            }
            
            if ( prev_time.minutes != time_date.minutes )
            {
                prev_time.minutes = time_date.minutes;
            
                ByteToStr( time_date.minutes, tx_tmp );
                Ltrim( tx_tmp );

                g2c3g_send_data_ref( &g2c_minRef[ 0 ], &tx_tmp[ 0 ] );
                g2c3g_send_cmd( &G2C3G_AT_PUB[ 0 ] );
            }

            if ( prev_time.monthday != time_date.monthday )
            {
                /** DATE UPDATE */
                prev_time.monthday = time_date.monthday;
                
                ByteToStr( time_date.monthday, tx_tmp );
                Ltrim( tx_tmp );

                g2c3g_send_data_ref( &g2c_dayRef[ 0 ], &tx_tmp[ 0 ] );
                g2c3g_send_cmd( &G2C3G_AT_PUB[ 0 ] );
                
                if ( prev_time.month != time_date.month )
                {
                    prev_time.month = time_date.month;
                    
                    ByteToStr( time_date.month, tx_tmp );
                    Ltrim( tx_tmp );
                    
                    g2c3g_send_data_ref( &g2c_monthRef[ 0 ], &tx_tmp[ 0 ] );
                    g2c3g_send_cmd( &G2C3G_AT_PUB[ 0 ] );
                }
                
                if ( prev_time.year != time_date.year )
                {
                    prev_time.year = time_date.year;
                    
                    ByteToStr( time_date.year, tx_tmp );
                    Ltrim( tx_tmp );
                    
                    g2c3g_send_data_ref( &g2c_yearRef[ 0 ], &tx_tmp[ 0 ] );
                    g2c3g_send_cmd( &G2C3G_AT_PUB[ 0 ] );
                }
                
                if ( prev_time.weekdays != time_date.weekdays )
                {
                    prev_time.weekdays = time_date.weekdays;
                    
                    ByteToStr( time_date.weekdays, tx_tmp );
                    Ltrim( tx_tmp );
                    
                    g2c3g_send_data_ref( &g2c_weekRef[ 0 ], &tx_tmp[ 0 ] );
                    g2c3g_send_cmd( &G2C3G_AT_PUB[ 0 ] );
                }
            }

            humidity = temphum8_getHumidity( );

            if ( prev_hum != humidity )
            {
                /** HUMIDITY UPDATE */
                prev_hum = humidity;
                
                ByteToStr( humidity, tx_tmp );
                Ltrim( tx_tmp );
                
                g2c3g_send_data_ref( &g2c_humRef[ 0 ], &tx_tmp[ 0 ] );
                g2c3g_send_cmd( &G2C3G_AT_PUB[ 0 ] );
            }

            temperature = temphum8_getTemperature( _TEMPHUM8_TEMPERATURE_IN_CELSIUS );

            if ( prev_temp != temperature )
            {
                /** TEMPERATURE UPDATE */
                prev_temp = temperature;
                
                ShortToStr( temperature, tx_tmp );
                Ltrim( tx_tmp );

                g2c3g_send_data_ref( &g2c_tempRef[ 0 ], &tx_tmp[ 0 ] );
                g2c3g_send_cmd( &G2C3G_AT_PUB[ 0 ] );
            }

            ambient_light = ambient6_getAmbientLight( );

            if ( prev_light != ambient_light )
            {
                /** AMBIENT LIGHT UPDATE */
                prev_light = ambient_light;
                
                WordToStr( ambient_light, tx_tmp );
                Ltrim( tx_tmp );
                
                g2c3g_send_data_ref( &g2c_lightRef[ 0 ], &tx_tmp[ 0 ] );
                g2c3g_send_cmd( &G2C3G_AT_PUB[ 0 ] );
            }
        }
    }

    /** STOP MEASUREMENT */
    if ( ( state == 0 ) && ( oldstate == 1 ) )
    {
        oldstate = 0;
    }
}

void main( )
{
    systemInit( );
    applicationInit( );

    while ( 1 )
    {
        applicationTask( );
    }
}

/* -------------------------------------------------------------------------- */